@@ -1,4 +1,5 @@
 #!/bin/sh
+set -e
 
 # "When we build software, it's a good idea to have a reliable method
 # for getting an executable from it. We want any two reconstructions
@@ -32,7 +33,7 @@
 #                 endlessly for a programmer to help it out with input
 #                 on *DEBUG-IO*
 #   "lisp -batch" to use an existing CMU CL binary as a cross-compilation host
-#   "lisp -noinit -batch" 
+#   "lisp -noinit -batch"
 #                 to use an existing CMU CL binary as a cross-compilation host
 #                 when you have weird things in your .cmucl-init file
 #   "openmcl --batch"
@@ -49,18 +50,23 @@
 # optimizations (especially specializable arrays) that it doesn't
 # know how to implement how in a portable way. (Or maybe that wouldn't
 # require a second pass, just testing at build-the-cross-compiler time
-# whether the cross-compilation host returns suitable values from 
+# whether the cross-compilation host returns suitable values from
 # UPGRADED-ARRAY-ELEMENT-TYPE?)
 
 LANG=C
 LC_ALL=C
-export LANG
-export LC_ALL
+export LANG LC_ALL
 
 build_started=`date`
 echo "//starting build: $build_started"
 
-SBCL_XC_HOST="${1:-sbcl --disable-debugger --userinit /dev/null --sysinit /dev/null}"
+if [ "$OSTYPE" = "cygwin" -o "$OSTYPE" = "msys" ] ; then
+    DEVNULL=NUL
+else
+    DEVNULL=/dev/null
+fi
+SBCL_XC_HOST="${1:-sbcl --userinit $DEVNULL --sysinit $DEVNULL}"
+export DEVNULL
 export SBCL_XC_HOST
 echo //SBCL_XC_HOST=\"$SBCL_XC_HOST\"
 
@@ -70,7 +76,10 @@ find_gnumake
 # If you're cross-compiling, you should probably just walk through the
 # make-config.sh script by hand doing the right thing on both the host
 # and target machines.
-sh make-config.sh || exit 1
+sh make-config.sh
+
+# Enforce the source policy for no bogus whitespace
+tools-for-build/canonicalize-whitespace
 
 # The make-host-*.sh scripts are run on the cross-compilation host,
 # and the make-target-*.sh scripts are run on the target machine. In
@@ -84,7 +93,7 @@ sh make-config.sh || exit 1
 #     identify the target architecture).
 #   On the host system:
 #     SBCL_XC_HOST=<whatever> sh make-host-1.sh
-#   Copy src/runtime/genesis/*.h from the host system to the target 
+#   Copy src/runtime/genesis/*.h from the host system to the target
 #     system.
 #   On the target system:
 #     sh make-target-1.sh
@@ -99,34 +108,17 @@ sh make-config.sh || exit 1
 # Or, if you can set up the files somewhere shared (with NFS, AFS, or
 # whatever) between the host machine and the target machine, the basic
 # procedure above should still work, but you can skip the "copy" steps.
-time sh make-host-1.sh   || exit 1
-time sh make-target-1.sh || exit 1
-time sh make-host-2.sh   || exit 1
-time sh make-target-2.sh || exit 1
-time sh make-target-contrib.sh || exit 1
-
-# Sometimes people used to see the "No tests failed." output from the last
-# DEFTEST in contrib self-tests and think that's all that is. So...
-HEADER_HAS_BEEN_PRINTED=false
-for dir in contrib/*
-do
-  if [ -d "$dir" -a -f "$dir/Makefile" -a ! -f "$dir/test-passed" ]; then
-      if $HEADER_HAS_BEEN_PRINTED; then
-	  echo > /dev/null
-      else
-	  echo "Failed contribs:"
-	  HEADER_HAS_BEEN_PRINTED=true
-      fi
-      echo "  `basename $dir`"
-  fi
-done
+time sh make-host-1.sh
+time sh make-target-1.sh
+time sh make-host-2.sh
+time sh make-target-2.sh
+time sh make-target-contrib.sh
 
 NCONTRIBS=`find contrib -name Makefile -print | wc -l`
 NPASSED=`find contrib -name test-passed -print | wc -l`
-
 echo
 echo "The build seems to have finished successfully, including $NPASSED (out of $NCONTRIBS)"
-echo "contributed modules. If you would like to run more extensive tests on" 
+echo "contributed modules. If you would like to run more extensive tests on"
 echo "the new SBCL, you can try:"
 echo
 echo "  cd tests && sh ./run-tests.sh"
@@ -141,7 +133,7 @@ echo "  cd doc/manual && make"
 echo
 echo "To install SBCL (more information in INSTALL):"
 echo
-echo "  sh install.sh" 
+echo "  sh install.sh"
 
 build_finished=`date`
 echo
