@@ -96,8 +96,8 @@
         *gc-inhibit* t
         *gc-pending* nil
         #!+sb-thread *stop-for-gc-pending* #!+sb-thread nil
-        sb!unix::*interrupts-enabled* t
-        sb!unix::*interrupt-pending* nil
+        *interrupts-enabled* t
+        *interrupt-pending* nil
         *break-on-signals* nil
         *maximum-error-depth* 10
         *current-error-depth* 0
@@ -117,6 +117,9 @@
   ;; this to be initialized, so we initialize it right away.
   (show-and-call !random-cold-init)
 
+  ;; Must be done before any non-opencoded array references are made.
+  (show-and-call !hairy-data-vector-reffer-init)
+
   (show-and-call !character-database-cold-init)
   (show-and-call !character-name-database-cold-init)
 
