@@ -120,8 +120,9 @@
 ;;; Given a signal context, return the floating point modes word in
 ;;; the same format as returned by FLOATING-POINT-MODES.
 ;;;
-;;; FIXME: surely this must be accessible somewhere under Darwin?
-#!-darwin
+;;; FIXME: surely this must be accessible somewhere under Darwin?  Or
+;;; under NetBSD?
+#!+linux
 (define-alien-routine ("os_context_fp_control" context-floating-point-modes)
     (sb!alien:unsigned 32)
   (context (* os-context-t)))
