@@ -14,26 +14,22 @@
 
 ;;;; utilities
 
-;; This stuff is performance critical and unwind-protect is too
-;; slow. And without the locking the next vector can get cyclic
-;; causing looping in a WITHOUT-GCING form, SHRINK-VECTOR can corrupt
-;; memory and who knows what else.
+;;; Without the locking the next vector can get cyclic causing
+;;; looping in a WITHOUT-GCING form, SHRINK-VECTOR can corrupt memory
+;;; and who knows what else.
+;;;
+;;; WITHOUT-GCING implies WITHOUT-INTERRUPTS.
 (defmacro with-spinlock-and-without-gcing ((spinlock) &body body)
   #!-sb-thread
   (declare (ignore spinlock))
-  (with-unique-names (old-gc-inhibit)
-    `(let ((,old-gc-inhibit *gc-inhibit*)
-           (*gc-inhibit* t))
-       (unwind-protect
-            (progn
-              #!+sb-thread
-              (sb!thread::get-spinlock ,spinlock)
-              ,@body)
-         #!+sb-thread
-         (sb!thread::release-spinlock ,spinlock)
-         (let ((*gc-inhibit* ,old-gc-inhibit))
-           ;; the test is racy, but it can err only on the overeager side
-           (sb!kernel::maybe-handle-pending-gc))))))
+  `(without-gcing
+     (unwind-protect
+          (progn
+            #!+sb-thread
+            (sb!thread::get-spinlock ,spinlock)
+            ,@body)
+       #!+sb-thread
+       (sb!thread::release-spinlock ,spinlock))))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defconstant max-hash sb!xc:most-positive-fixnum))
@@ -60,7 +56,16 @@
 #!-sb-fluid (declaim (inline equal-hash))
 (defun equal-hash (key)
   (declare (values hash (member t nil)))
-  (values (sxhash key) nil))
+  (typecase key
+    ;; For some types the definition of EQUAL implies a special hash
+    ((or string cons number bit-vector pathname)
+     (values (sxhash key) nil))
+    ;; Otherwise use an EQ hash, rather than SXHASH, since the values
+    ;; of SXHASH will be extremely badly distributed due to the
+    ;; requirements of the spec fitting badly with our implementation
+    ;; strategy.
+    (t
+     (eq-hash key))))
 
 #!-sb-fluid (declaim (inline eql-hash))
 (defun eql-hash (key)
@@ -71,7 +76,13 @@
 
 (defun equalp-hash (key)
   (declare (values hash (member t nil)))
-  (values (psxhash key) nil))
+  (typecase key
+    ;; Types requiring special treatment. Note that PATHNAME and
+    ;; HASH-TABLE are caught by the STRUCTURE-OBJECT test.
+    ((or array cons number character structure-object)
+     (values (psxhash key) nil))
+    (t
+     (eq-hash key))))
 
 (defun almost-primify (num)
   (declare (type index num))
@@ -323,6 +334,7 @@
     ;; the chains are first to last.
     (do ((i (1- new-size) (1- i)))
         ((zerop i))
+      (declare (type index/2 i))
       (let ((key (aref new-kv-vector (* 2 i)))
             (value (aref new-kv-vector (1+ (* 2 i)))))
         (cond ((and (eq key +empty-ht-slot+)
@@ -395,6 +407,7 @@
       (setf (aref index-vector i) 0))
     (do ((i (1- size) (1- i)))
         ((zerop i))
+      (declare (type index/2 i))
       (let ((key (aref kv-vector (* 2 i)))
             (value (aref kv-vector (1+ (* 2 i)))))
         (cond ((and (eq key +empty-ht-slot+)
@@ -433,7 +446,7 @@
          (length (length index-vector)))
     (do ((next (hash-table-needing-rehash table)))
         ((zerop next))
-      (declare (type index next))
+      (declare (type index/2 next))
       (let* ((key (aref kv-vector (* 2 next)))
              (hashing (pointer-hash key))
              (index (rem hashing length))
@@ -495,13 +508,13 @@
            (if (or eq-based (not hash-vector))
                (do ((next next (aref next-vector next)))
                    ((zerop next) (values default nil))
-                 (declare (type index next))
+                 (declare (type index/2 next))
                  (when (eq key (aref table (* 2 next)))
                    (setf (hash-table-cache hash-table) (* 2 next))
                    (return (values (aref table (1+ (* 2 next))) t))))
              (do ((next next (aref next-vector next)))
                  ((zerop next) (values default nil))
-               (declare (type index next))
+               (declare (type index/2 next))
                (when (and (= hashing (aref hash-vector next))
                           (funcall test-fun key (aref table (* 2 next))))
                  ;; Found.
@@ -549,7 +562,7 @@
                 (next-vector (hash-table-next-vector hash-table))
                 (hash-vector (hash-table-hash-vector hash-table))
                 (test-fun (hash-table-test-fun hash-table)))
-           (declare (type index index))
+           (declare (type index index next))
            (when (hash-table-weakness hash-table)
              (set-header-data kv-vector sb!vm:vector-valid-hashing-subtype))
            (cond ((or eq-based (not hash-vector))
@@ -560,7 +573,7 @@
                   ;; Search next-vector chain for a matching key.
                   (do ((next next (aref next-vector next)))
                       ((zerop next))
-                    (declare (type index next))
+                    (declare (type index/2 next))
                     (when (eq key (aref kv-vector (* 2 next)))
                       ;; Found, just replace the value.
                       (setf (hash-table-cache hash-table) (* 2 next))
@@ -570,7 +583,7 @@
                   ;; Search next-vector chain for a matching key.
                   (do ((next next (aref next-vector next)))
                       ((zerop next))
-                    (declare (type index next))
+                    (declare (type index/2 next))
                     (when (and (= hashing (aref hash-vector next))
                                (funcall test-fun key
                                         (aref kv-vector (* 2 next))))
@@ -581,6 +594,7 @@
 
            ;; Pop a KV slot off the free list
            (let ((free-kv-slot (hash-table-next-free-kv hash-table)))
+             (declare (type index/2 free-kv-slot))
              ;; Double-check for overflow.
              (aver (not (zerop free-kv-slot)))
              (setf (hash-table-next-free-kv hash-table)
@@ -633,8 +647,10 @@
             (next-vector (hash-table-next-vector hash-table))
             (hash-vector (hash-table-hash-vector hash-table))
             (test-fun (hash-table-test-fun hash-table)))
-       (declare (type index index next))
+       (declare (type index index)
+                (type index/2 next))
        (flet ((clear-slot (chain-vector prior-slot-location slot-location)
+                (declare (type index/2 slot-location))
                 ;; Mark slot as empty.
                 (setf (aref table (* 2 slot-location)) +empty-ht-slot+
                       (aref table (1+ (* 2 slot-location))) +empty-ht-slot+)
@@ -671,7 +687,7 @@
                 (do ((prior next next)
                      (next (aref next-vector next) (aref next-vector next)))
                     ((zerop next) nil)
-                  (declare (type index next))
+                  (declare (type index/2 next))
                   (when (and (= hashing (aref hash-vector next))
                              (funcall test-fun key (aref table (* 2 next))))
                     (return-from remhash
@@ -721,14 +737,16 @@
 (declaim (inline maphash))
 (defun maphash (function-designator hash-table)
   #!+sb-doc
-  "For each entry in HASH-TABLE, call the designated two-argument function
-   on the key and value of the entry. Return NIL."
+  "For each entry in HASH-TABLE, call the designated two-argument function on
+the key and value of the entry. Return NIL."
+  ;; This essentially duplicates WITH-HASH-TABLE-ITERATOR, so
+  ;; any changes here should be reflected there as well.
   (let ((fun (%coerce-callable-to-fun function-designator))
         (size (length (hash-table-next-vector hash-table))))
     (declare (type function fun))
     (do ((i 1 (1+ i)))
         ((>= i size))
-      (declare (type index i))
+      (declare (type index/2 i))
       (let* ((kv-vector (hash-table-table hash-table))
              (key (aref kv-vector (* 2 i)))
              (value (aref kv-vector (1+ (* 2 i)))))
