@@ -13,17 +13,22 @@
 
 (defun invoke-interruption (function)
   (without-interrupts
-    (sb!unix::reset-signal-mask)
-    (funcall function)))
+    ;; Reset signal mask: the C-side handler has blocked all
+    ;; deferrable interrupts before arranging return to lisp. This is
+    ;; safe because we can't get a pending interrupt before we unblock
+    ;; signals.
+    ;;
+    ;; FIXME: Should we not reset the _entire_ mask, just restore it
+    ;; to the state before we got the interrupt?
+    (reset-signal-mask)
+    ;; Tell INTERRUPT-THREAD it's ok to re-enable interrupts.
+    (let ((*in-interruption* t))
+      (funcall function))))
 
 (defmacro in-interruption ((&rest args) &body body)
   #!+sb-doc
   "Convenience macro on top of INVOKE-INTERRUPTION."
   `(invoke-interruption (lambda () ,@body) ,@args))
-
-;;; These should probably be somewhere, but I don't know where.
-(defconstant sig_dfl 0)
-(defconstant sig_ign 1)
 
 ;;;; system calls that deal with signals
 
@@ -73,13 +78,13 @@
     (without-gcing
       (let ((result (install-handler signal
                                      (case handler
-                                       (:default sig_dfl)
-                                       (:ignore sig_ign)
+                                       (:default sig-dfl)
+                                       (:ignore sig-ign)
                                        (t
                                         (sb!kernel:get-lisp-obj-address
                                          #'run-handler))))))
-        (cond ((= result sig_dfl) :default)
-              ((= result sig_ign) :ignore)
+        (cond ((= result sig-dfl) :default)
+              ((= result sig-ign) :ignore)
               (t (the (or function fixnum)
                    (sb!kernel:make-lisp-obj result))))))))
 
@@ -97,15 +102,8 @@
 ;;; *DEBUGGER-HOOK*, but we want SIGINT's BREAK to respect it, so that
 ;;; SIGINT in --disable-debugger mode will cleanly terminate the system
 ;;; (by respecting the *DEBUGGER-HOOK* established in that mode).
-(defun sigint-%break (format-string &rest format-arguments)
-  (flet ((break-it ()
-           (apply #'%break 'sigint format-string format-arguments)))
-    (sb!thread:interrupt-thread (sb!thread::foreground-thread) #'break-it)))
-
 (eval-when (:compile-toplevel :execute)
-  (sb!xc:defmacro define-signal-handler (name
-                                         what
-                                         &optional (function 'error))
+  (sb!xc:defmacro define-signal-handler (name what &optional (function 'error))
     `(defun ,name (signal info context)
        (declare (ignore signal info))
        (declare (type system-area-pointer context))
@@ -116,9 +114,7 @@
                     (with-alien ((context (* os-context-t) context))
                       (sap-int (sb!vm:context-pc context))))))))
 
-(define-signal-handler sigint-handler "interrupted" sigint-%break)
 (define-signal-handler sigill-handler "illegal instruction")
-(define-signal-handler sigtrap-handler "breakpoint/trap")
 #!-linux
 (define-signal-handler sigemt-handler "SIGEMT")
 (define-signal-handler sigbus-handler "bus error")
@@ -126,6 +122,18 @@
 #!-linux
 (define-signal-handler sigsys-handler "bad argument to a system call")
 
+(defun sigint-handler (signal info context)
+  (declare (ignore signal info))
+  (declare (type system-area-pointer context))
+  (/show "in Lisp-level SIGINT handler" (sap-int context))
+  (flet ((interrupt-it ()
+           (with-alien ((context (* os-context-t) context))
+             (%break 'sigint 'interactive-interrupt
+                     :context context
+                     :address (sap-int (sb!vm:context-pc context))))))
+    (sb!thread:interrupt-thread (sb!thread::foreground-thread)
+                                #'interrupt-it)))
+
 (defun sigalrm-handler (signal info context)
   (declare (ignore signal info context))
   (declare (type system-area-pointer context))
@@ -147,7 +155,6 @@
   (enable-interrupt sigint #'sigint-handler)
   (enable-interrupt sigterm #'sigterm-handler)
   (enable-interrupt sigill #'sigill-handler)
-  (enable-interrupt sigtrap #'sigtrap-handler)
   (enable-interrupt sigiot #'sigiot-handler)
   #!-linux
   (enable-interrupt sigemt #'sigemt-handler)
@@ -171,31 +178,3 @@
 ;;;   Magically converted by the compiler into a break instruction.
 (defun receive-pending-interrupt ()
   (receive-pending-interrupt))
-
-;;; stale code which I'm insufficiently motivated to test -- WHN 19990714
-#|
-;;;; WITH-ENABLED-INTERRUPTS
-
-(defmacro with-enabled-interrupts (interrupt-list &body body)
-  #!+sb-doc
-  "With-enabled-interrupts ({(interrupt function)}*) {form}*
-   Establish function as a handler for the Unix signal interrupt which
-   should be a number between 1 and 31 inclusive."
-  (let ((il (gensym))
-        (it (gensym)))
-    `(let ((,il NIL))
-       (unwind-protect
-           (progn
-             ,@(do* ((item interrupt-list (cdr item))
-                     (intr (caar item) (caar item))
-                     (ifcn (cadar item) (cadar item))
-                     (forms NIL))
-                    ((null item) (nreverse forms))
-                 (when (symbolp intr)
-                   (setq intr (symbol-value intr)))
-                 (push `(push `(,,intr ,(enable-interrupt ,intr ,ifcn)) ,il)
-                       forms))
-             ,@body)
-         (dolist (,it (nreverse ,il))
-           (enable-interrupt (car ,it) (cadr ,it)))))))
-|#
