@@ -19,7 +19,7 @@
 ;;; set the doc here because in early-thread FDOCUMENTATION is not
 ;;; available, yet
 #!+sb-doc
-(setf (sb!kernel:fdocumentation '*current-thread* 'variable)
+(setf (fdocumentation '*current-thread* 'variable)
       "Bound in each thread to the thread itself.")
 
 (defstruct (thread (:constructor %make-thread))
@@ -30,10 +30,12 @@ in future versions."
   %alive-p
   os-thread
   interruptions
-  (interruptions-lock (make-mutex :name "thread interruptions lock")))
+  (interruptions-lock (make-mutex :name "thread interruptions lock"))
+  result
+  (result-lock (make-mutex :name "thread result lock")))
 
 #!+sb-doc
-(setf (sb!kernel:fdocumentation 'thread-name 'function)
+(setf (fdocumentation 'thread-name 'function)
       "The name of the thread. Setfable.")
 
 (def!method print-object ((thread thread) stream)
@@ -57,10 +59,18 @@ in future versions."
 (defvar *all-threads* ())
 (defvar *all-threads-lock* (make-mutex :name "all threads lock"))
 
+(defmacro with-all-threads-lock (&body body)
+  #!-sb-thread
+  `(locally ,@body)
+  #!+sb-thread
+  `(without-interrupts
+     (with-mutex (*all-threads-lock*)
+       ,@body)))
+
 (defun list-all-threads ()
   #!+sb-doc
   "Return a list of the live threads."
-  (with-mutex (*all-threads-lock*)
+  (with-all-threads-lock
     (copy-list *all-threads*)))
 
 (declaim (inline current-thread-sap))
@@ -97,7 +107,7 @@ in future versions."
   (define-alien-routine "signal_interrupt_thread"
       integer (os-thread unsigned-long))
 
-  (define-alien-routine "block_blockable_signals"
+  (define-alien-routine "block_deferrable_signals"
       void)
 
   #!+sb-lutex
@@ -117,6 +127,9 @@ in future versions."
     (sb!alien:define-alien-routine ("lutex_lock" %lutex-lock)
         int (lutex unsigned-long))
 
+    (sb!alien:define-alien-routine ("lutex_trylock" %lutex-trylock)
+        int (lutex unsigned-long))
+
     (sb!alien:define-alien-routine ("lutex_unlock" %lutex-unlock)
         int (lutex unsigned-long))
 
@@ -130,7 +143,7 @@ in future versions."
     (defmacro with-lutex-address ((name lutex) &body body)
       `(let ((,name ,lutex))
          (with-pinned-objects (,name)
-           (let ((,name (sb!kernel:get-lisp-obj-address ,name)))
+           (let ((,name (get-lisp-obj-address ,name)))
              ,@body))))
 
     (defun make-lutex ()
@@ -150,88 +163,90 @@ in future versions."
     (declaim (inline futex-wait futex-wake))
 
     (sb!alien:define-alien-routine "futex_wait"
-        int (word unsigned-long) (old-value unsigned-long))
+        int (word unsigned-long) (old-value unsigned-long)
+        (to-sec long) (to-usec unsigned-long))
 
     (sb!alien:define-alien-routine "futex_wake"
         int (word unsigned-long) (n unsigned-long))))
 
 ;;; used by debug-int.lisp to access interrupt contexts
-#!-(and sb-fluid sb-thread) (declaim (inline sb!vm::current-thread-offset-sap))
+#!-(or sb-fluid sb-thread) (declaim (inline sb!vm::current-thread-offset-sap))
 #!-sb-thread
 (defun sb!vm::current-thread-offset-sap (n)
   (declare (type (unsigned-byte 27) n))
   (sap-ref-sap (alien-sap (extern-alien "all_threads" (* t)))
                (* n sb!vm:n-word-bytes)))
 
+#!+sb-thread
+(defun sb!vm::current-thread-offset-sap (n)
+  (declare (type (unsigned-byte 27) n))
+  (sb!vm::current-thread-offset-sap n))
+
 ;;;; spinlocks
+#!+sb-thread
+(define-structure-slot-compare-and-swap
+    compare-and-swap-spinlock-value
+    :structure spinlock
+    :slot value)
 
 (declaim (inline get-spinlock release-spinlock))
 
-;;; The bare 2 here and below are offsets of the slots in the struct.
-;;; There ought to be some better way to get these numbers
 (defun get-spinlock (spinlock)
   (declare (optimize (speed 3) (safety 0))
            #!-sb-thread
-           (ignore spinlock new-value))
+           (ignore spinlock))
   ;; %instance-set-conditional can test for 0 (which is a fixnum) and
   ;; store any value
   #!+sb-thread
   (loop until
-        (eql (sb!vm::%instance-set-conditional spinlock 2 0 1) 0)))
+       (eql 0 (compare-and-swap-spinlock-value spinlock 0 1)))
+  t)
 
 (defun release-spinlock (spinlock)
   (declare (optimize (speed 3) (safety 0))
            #!-sb-thread (ignore spinlock))
   ;; %instance-set-conditional cannot compare arbitrary objects
-  ;; meaningfully, so
-  ;; (sb!vm::%instance-set-conditional spinlock 2 our-value 0)
+  ;; meaningfully, so (compare-and-swap-spinlock-value our-value 0)
   ;; does not work for bignum thread ids.
   #!+sb-thread
-  (sb!vm::%instance-set spinlock 2 0))
-
-(defmacro with-spinlock ((spinlock) &body body)
-  (sb!int:with-unique-names (lock)
-    `(let ((,lock ,spinlock))
-      (get-spinlock ,lock)
-      (unwind-protect
-           (progn ,@body)
-        (release-spinlock ,lock)))))
+  (setf (spinlock-value spinlock) 0)
+  nil)
 
 ;;;; mutexes
 
 #!+sb-doc
-(setf (sb!kernel:fdocumentation 'make-mutex 'function)
+(setf (fdocumentation 'make-mutex 'function)
       "Create a mutex."
-      (sb!kernel:fdocumentation 'mutex-name 'function)
+      (fdocumentation 'mutex-name 'function)
       "The name of the mutex. Setfable."
-      (sb!kernel:fdocumentation 'mutex-value 'function)
+      (fdocumentation 'mutex-value 'function)
       "The value of the mutex. NIL if the mutex is free. Setfable.")
 
 #!+(and sb-thread (not sb-lutex))
 (progn
-  (declaim (inline mutex-value-address))
-  (defun mutex-value-address (mutex)
-    (declare (optimize (speed 3)))
-    (sb!ext:truly-the
-     sb!vm:word
-     (+ (sb!kernel:get-lisp-obj-address mutex)
-        (- (* 3 sb!vm:n-word-bytes) sb!vm:instance-pointer-lowtag)))))
+  (define-structure-slot-addressor mutex-value-address
+      :structure mutex
+      :slot value)
+  (define-structure-slot-compare-and-swap
+      compare-and-swap-mutex-value
+      :structure mutex
+      :slot value))
 
-(defun get-mutex (mutex &optional (new-value *current-thread*) (wait-p t))
+(defun get-mutex (mutex &optional (new-value *current-thread*) (waitp t))
   #!+sb-doc
   "Acquire MUTEX, setting it to NEW-VALUE or some suitable default
-value if NIL.  If WAIT-P is non-NIL and the mutex is in use, sleep
-until it is available"
+value if NIL. If WAITP is non-NIL and the mutex is in use, sleep
+until it is available."
   (declare (type mutex mutex) (optimize (speed 3)))
   (/show0 "Entering GET-MUTEX")
   (unless new-value
     (setq new-value *current-thread*))
   #!-sb-thread
-  (let ((old-value (mutex-value mutex)))
-    (when (and old-value wait-p)
-      (error "In unithread mode, mutex ~S was requested with WAIT-P ~S and ~
+  (let ((old (mutex-value mutex)))
+    (when (and old waitp)
+      (error "In unithread mode, mutex ~S was requested with WAITP ~S and ~
               new-value ~S, but has already been acquired (with value ~S)."
-             mutex wait-p new-value old-value))
+             mutex waitp new-value old))
     (setf (mutex-value mutex) new-value)
     t)
   #!+sb-thread
@@ -241,23 +256,30 @@ until it is available"
       (format *debug-io* "Thread: ~A~%" *current-thread*)
       (sb!debug:backtrace most-positive-fixnum *debug-io*)
       (force-output *debug-io*))
-    ;; FIXME: sb-lutex and (not wait-p)
+    ;; FIXME: Lutexes do not currently support deadlines, as at least
+    ;; on Darwin pthread_foo_timedbar functions are not supported:
+    ;; this means that we probably need to use the Carbon multiprocessing
+    ;; functions on Darwin.
     #!+sb-lutex
-    (when wait-p
-      (with-lutex-address (lutex (mutex-lutex mutex))
-        (%lutex-lock lutex))
+    (when (zerop (with-lutex-address (lutex (mutex-lutex mutex))
+                   (if waitp
+                       (%lutex-lock lutex)
+                       (%lutex-trylock lutex))))
       (setf (mutex-value mutex) new-value))
     #!-sb-lutex
     (let (old)
-      (loop
-         (unless
-             (setf old (sb!vm::%instance-set-conditional mutex 2 nil
-                                                         new-value))
-           (return t))
-         (unless wait-p (return nil))
-         (with-pinned-objects (mutex old)
-           (futex-wait (mutex-value-address mutex)
-                       (sb!kernel:get-lisp-obj-address old)))))))
+      (when (and (setf old (compare-and-swap-mutex-value mutex nil new-value))
+                 waitp)
+        (loop while old
+              do (multiple-value-bind (to-sec to-usec) (decode-timeout nil)
+                   (when (= 1 (with-pinned-objects (mutex old)
+                                (futex-wait (mutex-value-address mutex)
+                                            (get-lisp-obj-address old)
+                                            (or to-sec -1)
+                                            (or to-usec 0))))
+                     (signal-deadline)))
+              (setf old (compare-and-swap-mutex-value mutex nil new-value))))
+      (not old))))
 
 (defun release-mutex (mutex)
   #!+sb-doc
@@ -291,18 +313,13 @@ this mutex."
   (%make-waitqueue :name name))
 
 #!+sb-doc
-(setf (sb!kernel:fdocumentation 'waitqueue-name 'function)
+(setf (fdocumentation 'waitqueue-name 'function)
       "The name of the waitqueue. Setfable.")
 
 #!+(and sb-thread (not sb-lutex))
-(progn
-  (declaim (inline waitqueue-data-address))
-  (defun waitqueue-data-address (waitqueue)
-    (declare (optimize (speed 3)))
-    (sb!ext:truly-the
-     sb!vm:word
-     (+ (sb!kernel:get-lisp-obj-address waitqueue)
-        (- (* 3 sb!vm:n-word-bytes) sb!vm:instance-pointer-lowtag)))))
+(define-structure-slot-addressor waitqueue-data-address
+    :structure waitqueue
+    :slot data)
 
 (defun condition-wait (queue mutex)
   #!+sb-doc
@@ -333,10 +350,15 @@ time we reacquire MUTEX and return to the caller."
            ;; manages to grab MUTEX and call CONDITION-NOTIFY during
            ;; this comment, it will change queue->data, and so
            ;; futex-wait returns immediately instead of sleeping.
-           ;; Ergo, no lost wakeup
-           (with-pinned-objects (queue me)
-             (futex-wait (waitqueue-data-address queue)
-                         (sb!kernel:get-lisp-obj-address me))))
+           ;; Ergo, no lost wakeup. We may get spurious wakeups,
+           ;; but that's ok.
+           (multiple-value-bind (to-sec to-usec) (decode-timeout nil)
+             (when (= 1 (with-pinned-objects (queue me)
+                          (futex-wait (waitqueue-data-address queue)
+                                      (get-lisp-obj-address me)
+                                      (or to-sec -1) ;; our way if saying "no timeout"
+                                      (or to-usec 0))))
+               (signal-deadline))))
       ;; If we are interrupted while waiting, we should do these things
       ;; before returning.  Ideally, in the case of an unhandled signal,
       ;; we should do them before entering the debugger, but this is
@@ -392,7 +414,7 @@ time we reacquire MUTEX and return to the caller."
   "Create a semaphore with the supplied COUNT."
   (%make-semaphore :name name :count count))
 
-(setf (sb!kernel:fdocumentation 'semaphore-name 'function)
+(setf (fdocumentation 'semaphore-name 'function)
       "The name of the semaphore. Setfable.")
 
 (defun wait-on-semaphore (sem)
@@ -467,16 +489,22 @@ this semaphore, then N of them is woken up."
 ;;; Remove thread from its session, if it has one.
 #!+sb-thread
 (defun handle-thread-exit (thread)
-  (with-mutex (*all-threads-lock*)
-    (/show0 "HANDLING THREAD EXIT")
-    #!+sb-lutex
-    (when (thread-interruptions-lock thread)
-      (/show0 "FREEING MUTEX LUTEX")
-      (with-lutex-address (lutex (mutex-lutex (thread-interruptions-lock thread)))
-        (%lutex-destroy lutex)))
-    (setq *all-threads* (delete thread *all-threads*)))
-  (when *session*
-    (%delete-thread-from-session thread *session*)))
+  (/show0 "HANDLING THREAD EXIT")
+  ;; We're going down, can't handle interrupts sanely anymore.
+  ;; GC remains enabled.
+  (block-deferrable-signals)
+  ;; Lisp-side cleanup
+  (with-all-threads-lock
+    (setf (thread-%alive-p thread) nil)
+    (setf (thread-os-thread thread) nil)
+    (setq *all-threads* (delete thread *all-threads*))
+    (when *session*
+      (%delete-thread-from-session thread *session*)))
+  #!+sb-lutex
+  (without-gcing
+    (/show0 "FREEING MUTEX LUTEX")
+    (with-lutex-address (lutex (mutex-lutex (thread-interruptions-lock thread)))
+      (%lutex-destroy lutex))))
 
 (defun terminate-session ()
   #!+sb-doc
@@ -582,7 +610,8 @@ have the foreground next."
 (defun make-thread (function &key name)
   #!+sb-doc
   "Create a new thread of NAME that runs FUNCTION. When the function
-returns the thread exits."
+returns the thread exits. The return values of FUNCTION are kept
+around and can be retrieved by JOIN-THREAD."
   #!-sb-thread (declare (ignore function name))
   #!-sb-thread (error "Not supported in unithread builds.")
   #!+sb-thread
@@ -604,9 +633,10 @@ returns the thread exits."
             ;; least accessible to users to secure their own libraries.
             ;;   --njf, 2006-07-15
             (let ((*current-thread* thread)
-                  (sb!kernel::*restart-clusters* nil)
-                  (sb!kernel::*handler-clusters* nil)
-                  (sb!kernel::*condition-restarts* nil)
+                  (*restart-clusters* nil)
+                  (*handler-clusters* nil)
+                  (*condition-restarts* nil)
+                  (sb!impl::*step-out* nil)
                   ;; internal printer variables
                   (sb!impl::*previous-case* nil)
                   (sb!impl::*previous-readtable-case* nil)
@@ -615,37 +645,33 @@ returns the thread exits."
                   (sb!impl::*internal-symbol-output-fun* nil)
                   (sb!impl::*descriptor-handlers* nil)) ; serve-event
               (setf (thread-os-thread thread) (current-thread-sap-id))
-              (with-mutex (*all-threads-lock*)
-                (push thread *all-threads*))
-              (with-session-lock (*session*)
-                (push thread (session-threads *session*)))
-              (setf (thread-%alive-p thread) t)
-              (signal-semaphore setup-sem)
-              ;; can't use handling-end-of-the-world, because that flushes
-              ;; output streams, and we don't necessarily have any (or we
-              ;; could be sharing them)
-              (catch 'sb!impl::toplevel-catcher
-                (catch 'sb!impl::%end-of-the-world
-                  (with-simple-restart
-                      (terminate-thread
-                       (format nil
-                               "~~@<Terminate this thread (~A)~~@:>"
-                               *current-thread*))
-                    (unwind-protect
-                         (progn
-                           ;; now that most things have a chance to
-                           ;; work properly without messing up other
-                           ;; threads, it's time to enable signals
-                           (sb!unix::reset-signal-mask)
-                           (funcall real-function))
-                      ;; we're going down, can't handle
-                      ;; interrupts sanely anymore
-                      (let ((sb!impl::*gc-inhibit* t))
-                        (block-blockable-signals)
-                        (setf (thread-%alive-p thread) nil)
-                        (setf (thread-os-thread thread) nil)
-                        ;; and remove what can be the last
-                        ;; reference to this thread
+              (with-mutex ((thread-result-lock thread))
+                (with-all-threads-lock
+                  (push thread *all-threads*))
+                (with-session-lock (*session*)
+                  (push thread (session-threads *session*)))
+                (setf (thread-%alive-p thread) t)
+                (signal-semaphore setup-sem)
+                ;; can't use handling-end-of-the-world, because that flushes
+                ;; output streams, and we don't necessarily have any (or we
+                ;; could be sharing them)
+                (catch 'sb!impl::toplevel-catcher
+                  (catch 'sb!impl::%end-of-the-world
+                    (with-simple-restart
+                        (terminate-thread
+                         (format nil
+                                 "~~@<Terminate this thread (~A)~~@:>"
+                                 *current-thread*))
+                      (unwind-protect
+                           (progn
+                             ;; now that most things have a chance to
+                             ;; work properly without messing up other
+                             ;; threads, it's time to enable signals
+                             (sb!unix::reset-signal-mask)
+                             (setf (thread-result thread)
+                                   (cons t
+                                         (multiple-value-list
+                                          (funcall real-function)))))
                         (handle-thread-exit thread)))))))
             (values))))
     ;; Keep INITIAL-FUNCTION pinned until the child thread is
@@ -653,12 +679,38 @@ returns the thread exits."
     (with-pinned-objects (initial-function)
       (let ((os-thread
              (%create-thread
-              (sb!kernel:get-lisp-obj-address initial-function))))
+              (get-lisp-obj-address initial-function))))
         (when (zerop os-thread)
           (error "Can't create a new thread"))
         (wait-on-semaphore setup-sem)
         thread))))
 
+(define-condition join-thread-error (error)
+  ((thread :reader join-thread-error-thread :initarg :thread))
+  #!+sb-doc
+  (:documentation "Joining thread failed.")
+  (:report (lambda (c s)
+             (format s "Joining thread failed: thread ~A ~
+                        has not returned normally."
+                     (join-thread-error-thread c)))))
+
+#!+sb-doc
+(setf (fdocumentation 'join-thread-error-thread 'function)
+      "The thread that we failed to join.")
+
+(defun join-thread (thread &key (default nil defaultp))
+  #!+sb-doc
+  "Suspend current thread until THREAD exits. Returns the result
+values of the thread function. If the thread does not exit normally,
+return DEFAULT if given or else signal JOIN-THREAD-ERROR."
+  (with-mutex ((thread-result-lock thread))
+    (cond ((car (thread-result thread))
+           (values-list (cdr (thread-result thread))))
+          (defaultp
+           default)
+          (t
+           (error 'join-thread-error :thread thread)))))
+
 (defun destroy-thread (thread)
   #!+sb-doc
   "Deprecated. Same as TERMINATE-THREAD."
@@ -673,7 +725,7 @@ returns the thread exits."
                      (interrupt-thread-error-thread c)))))
 
 #!+sb-doc
-(setf (sb!kernel:fdocumentation 'interrupt-thread-error-thread 'function)
+(setf (fdocumentation 'interrupt-thread-error-thread 'function)
       "The thread that was not interrupted.")
 
 (defmacro with-interruptions-lock ((thread) &body body)
@@ -681,13 +733,15 @@ returns the thread exits."
      (with-mutex ((thread-interruptions-lock ,thread))
        ,@body)))
 
-;; Called from the signal handler.
+;; Called from the signal handler in C.
 (defun run-interruption ()
   (in-interruption ()
     (loop
        (let ((interruption (with-interruptions-lock (*current-thread*)
                              (pop (thread-interruptions *current-thread*)))))
          (if interruption
+             ;; This is safe because it's the IN-INTERRUPTION that
+             ;; has disabled interrupts.
              (with-interrupts
                (funcall interruption))
              (return))))))
@@ -707,21 +761,29 @@ nature: if you interrupt a thread that was holding important locks
 then do something that turns out to need those locks, you probably
 won't like the effect."
   #!-sb-thread (declare (ignore thread))
-  ;; not quite perfect, because it does not take WITHOUT-INTERRUPTS
-  ;; into account
-  #!-sb-thread
-  (funcall function)
-  #!+sb-thread
-  (if (eq thread *current-thread*)
-      (funcall function)
-      (let ((os-thread (thread-os-thread thread)))
-        (cond ((not os-thread)
-               (error 'interrupt-thread-error :thread thread))
-              (t
-               (with-interruptions-lock (thread)
-                 (push function (thread-interruptions thread)))
-               (when (minusp (signal-interrupt-thread os-thread))
-                 (error 'interrupt-thread-error :thread thread)))))))
+  (flet ((interrupt-self ()
+           ;; *IN-INTERRUPTION* is true IFF we're being called as an
+           ;; interruption without an intervening WITHOUT-INTERRUPTS,
+           ;; in which case it is safe to enable interrupts. Otherwise
+           ;; interrupts are either already enabled, or there is an outer
+           ;; WITHOUT-INTERRUPTS we know nothing about, which makes it
+           ;; unsafe to enable interrupts.
+           (if *in-interruption*
+               (with-interrupts (funcall function))
+               (funcall function))))
+    #!-sb-thread
+    (interrupt-self)
+    #!+sb-thread
+    (if (eq thread *current-thread*)
+        (interrupt-self)
+        (let ((os-thread (thread-os-thread thread)))
+          (cond ((not os-thread)
+                 (error 'interrupt-thread-error :thread thread))
+                (t
+                 (with-interruptions-lock (thread)
+                   (push function (thread-interruptions thread)))
+                 (when (minusp (signal-interrupt-thread os-thread))
+                   (error 'interrupt-thread-error :thread thread))))))))
 
 (defun terminate-thread (thread)
   #!+sb-doc
@@ -753,4 +815,19 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
                                (* sb!vm:n-word-bytes index))))
     (if (eql tl-val sb!vm::no-tls-value-marker-widetag)
         (sb!vm::symbol-global-value symbol)
-        (sb!kernel:make-lisp-obj tl-val))))
+        (make-lisp-obj tl-val))))
+
+(defun sb!vm::locked-symbol-global-value-add (symbol-name delta)
+  (sb!vm::locked-symbol-global-value-add symbol-name delta))
+
+;;; Stepping
+
+(defun thread-stepping ()
+  (make-lisp-obj
+   (sap-ref-word (current-thread-sap)
+                 (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes))))
+
+(defun (setf thread-stepping) (value)
+  (setf (sap-ref-word (current-thread-sap)
+                      (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes))
+        (get-lisp-obj-address value)))
