@@ -20,6 +20,7 @@
     sb!kernel::undefined-alien-variable-error
     sb!kernel::undefined-alien-function-error
     sb!kernel::memory-fault-error
+    sb!kernel::unhandled-trap-error
     sb!di::handle-breakpoint
     sb!di::handle-single-step-trap
     fdefinition-object
@@ -58,6 +59,12 @@
     #!-sb-thread
     *stepping*
 
+    ;; Dispatch tables for generic array access
+    sb!impl::*data-vector-reffers*
+    sb!impl::*data-vector-setters*
+    sb!impl::*data-vector-reffers/check-bounds*
+    sb!impl::*data-vector-setters/check-bounds*
+
     ;; hash table weaknesses
     :key
     :value
