@@ -20,11 +20,12 @@
            complex-rational-p complex-float-p complex-single-float-p
            complex-double-float-p #!+long-float complex-long-float-p
            complex-vector-p
-           base-char-p %standard-char-p %instancep
+           base-char-p %standard-char-p %instancep %other-pointer-p
            base-string-p simple-base-string-p
            #!+sb-unicode character-string-p
            #!+sb-unicode simple-character-string-p
            array-header-p
+           sequencep extended-sequence-p
            simple-array-p simple-array-nil-p vector-nil-p
            simple-array-unsigned-byte-2-p
            simple-array-unsigned-byte-4-p simple-array-unsigned-byte-7-p
@@ -199,11 +200,17 @@
 
 (defknown %raw-bits (t fixnum) sb!vm:word
   (foldable flushable))
+#!+x86
+(defknown %raw-bits-with-offset (t fixnum fixnum) sb!vm:word
+  (flushable always-translatable))
 (defknown (%set-raw-bits) (t fixnum sb!vm:word) sb!vm:word
   (unsafe))
-;; These two are mostly used for bit-bashing operations.
+#!+x86
+(defknown (%set-raw-bits-with-offset) (t fixnum fixnum sb!vm:word) sb!vm:word
+  (unsafe always-translatable))
+;;; These two are mostly used for bit-bashing operations.
 (defknown %vector-raw-bits (t fixnum) sb!vm:word
-  (foldable flushable))
+  (flushable))
 (defknown (%set-vector-raw-bits) (t fixnum sb!vm:word) sb!vm:word
   (unsafe))
 
@@ -283,10 +290,17 @@
 
 (defknown %bignum-ref (bignum-type bignum-index) bignum-element-type
   (flushable))
+#!+(or x86 x86-64)
+(defknown %bignum-ref-with-offset (bignum-type bignum-index (signed-byte 24))
+  bignum-element-type (flushable always-translatable))
 
 (defknown %bignum-set (bignum-type bignum-index bignum-element-type)
   bignum-element-type
   (unsafe))
+#!+(or x86 x86-64)
+(defknown %bignum-set-with-offset
+  (bignum-type bignum-index (signed-byte 24) bignum-element-type)
+  bignum-element-type (unsafe always-translatable))
 
 (defknown %digit-0-or-plusp (bignum-element-type) boolean
   (foldable flushable movable))
