@@ -127,7 +127,9 @@
   unused
   pseudo-atomic
   object-not-list
-  object-not-instance)
+  object-not-instance
+  single-step-around
+  single-step-before)
 
 (defenum (:prefix trace-table-)
   normal
