@@ -149,7 +149,6 @@
               (inst lr temp (logior (ash (1- size) n-widetag-bits) closure-header-widetag))))
         ;;; should this be closure-fun-slot instead of 0?
         (storew temp result 0 fun-pointer-lowtag)
-        (storew result result closure-self-slot fun-pointer-lowtag)
         (storew function result closure-fun-slot fun-pointer-lowtag)))))
 
 ;;; The compiler likes to be able to directly make value cells.
@@ -173,6 +172,12 @@
   (:generator 1
     (inst li result unbound-marker-widetag)))
 
+(define-vop (make-funcallable-instance-tramp)
+  (:args)
+  (:results (result :scs (any-reg)))
+  (:generator 1
+    (inst lr result (make-fixup "funcallable_instance_tramp" :foreign))))
+
 (define-vop (fixed-alloc)
   (:args)
   (:info name words type lowtag)
