@@ -67,7 +67,7 @@
       (:little-endian
        `(inst lbz ,n-target ,n-source ,n-offset))
       (:big-endian
-       `(inst lbz ,n-target ,n-source (+ ,n-offset 3))))))
+       `(inst lbz ,n-target ,n-source (+ ,n-offset (1- n-word-bytes)))))))
 
 ;;; Macros to handle the fact that we cannot use the machine native call and
 ;;; return instructions.
@@ -340,13 +340,11 @@
        (inst andi. ,flag-tn alloc-tn 7)
        (inst twi :ne ,flag-tn 0))))
 
-
-
 (defmacro sb!sys::with-pinned-objects ((&rest objects) &body body)
   "Arrange with the garbage collector that the pages occupied by
 OBJECTS will not be moved in memory for the duration of BODY.
 Useful for e.g. foreign calls where another thread may trigger
 garbage collection.  This is currently implemented by disabling GC"
-  (declare (ignore objects))            ;should we eval these for side-effect?
+  (declare (ignore objects))            ; should we eval these for side-effect?
   `(without-gcing
     ,@body))
