@@ -35,9 +35,11 @@
 ;;; addressable object
 (def!constant n-byte-bits 8)
 
-;;; The size in bytes of the GENCGC pages. Should be a multiple of the
-;;; architecture code size.
-(def!constant gencgc-page-size 4096)
+;;; The minimum immediate offset in a memory-referencing instruction.
+(def!constant minimum-immediate-offset (- (expt 2 31)))
+
+;;; The maximum immediate offset in a memory-referencing instruction.
+(def!constant maximum-immediate-offset (1- (expt 2 31)))
 
 (def!constant float-sign-shift 31)
 
@@ -101,19 +103,21 @@
 ;;; would be possible, but probably not worth the time and code bloat
 ;;; it would cause. -- JES, 2005-12-11
 
-(def!constant read-only-space-start     #x20000000)
-(def!constant read-only-space-end       #x27ff0000)
+(progn
+  (def!constant read-only-space-start     #x20000000)
+  (def!constant read-only-space-end       #x200ff000)
 
-(def!constant static-space-start        #x40000000)
-(def!constant static-space-end          #x47fff000)
+  (def!constant static-space-start        #x20100000)
+  (def!constant static-space-end          #x201ff000)
 
-(def!constant dynamic-space-start   #x1000000000)
-(def!constant dynamic-space-end     #x11ffff0000)
+  (def!constant dynamic-space-start   #x1000000000)
+  (def!constant dynamic-space-end     #x11ffff0000)
 
-(def!constant linkage-table-space-start #x60000000)
-(def!constant linkage-table-space-end   #x63fff000)
+  (def!constant linkage-table-space-start #x20200000)
+  (def!constant linkage-table-space-end   #x202ff000)
+
+  (def!constant linkage-table-entry-size 16))
 
-(def!constant linkage-table-entry-size 16)
 
 ;;;; other miscellaneous constants
 
