@@ -62,7 +62,7 @@ int personality (unsigned long);
 
 size_t os_vm_page_size;
 
-#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_LUTEX)
+#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_LUTEX) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
 #include <sys/syscall.h>
 #include <unistd.h>
 #include <errno.h>
@@ -82,21 +82,30 @@ static inline int sys_futex (void *futex, int op, int val, struct timespec *rel)
 }
 
 int
-futex_wait(int *lock_word, int oldval)
+futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
 {
-    int t;
-  again:
+  struct timespec timeout;
+  int t;
+
+ again:
+  if (sec<0) {
     t = sys_futex(lock_word,FUTEX_WAIT,oldval, 0);
-
-    /* Interrupted FUTEX_WAIT calls may return early.
-     *
-     * If someone manages to wake the futex while we're spinning
-     * around it, we will just return with -1 and errno EWOULDBLOCK,
-     * because the value has changed, so that's ok. */
-    if (t != 0 && errno == EINTR)
-        goto again;
-
-    return t;
+  }
+  else {
+    timeout.tv_sec = sec;
+    timeout.tv_nsec = usec * 1000;
+    t = sys_futex(lock_word,FUTEX_WAIT,oldval, &timeout);
+  }
+  if (t==0)
+      return 0;
+  else if (errno==ETIMEDOUT)
+      return 1;
+  else if (errno==EINTR)
+      /* spurious wakeup from interrupt */
+      goto again;
+  else
+      /* EWOULDBLOCK and others, need to check the lock */
+      return -1;
 }
 
 int
@@ -145,7 +154,7 @@ os_init(char *argv[], char *envp[])
 {
     /* Conduct various version checks: do we have enough mmap(), is
      * this a sparc running 2.2, can we do threads? */
-#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_LUTEX)
+#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_LUTEX) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
     int *futex=0;
 #endif
     struct utsname name;
@@ -171,8 +180,8 @@ os_init(char *argv[], char *envp[])
 #endif
     }
 #ifdef LISP_FEATURE_SB_THREAD
-#if !defined(LISP_FEATURE_SB_LUTEX)
-    futex_wait(futex,-1);
+#if !defined(LISP_FEATURE_SB_LUTEX) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+    futex_wait(futex,-1,-1,0);
     if(errno==ENOSYS) {
        lose("This version of SBCL is compiled with threading support, but your kernel\n"
             "is too old to support this. Please use a more recent kernel or\n"
@@ -185,7 +194,12 @@ os_init(char *argv[], char *envp[])
             "LD_ASSUME_KERNEL\n");
     }
 #endif
-    os_vm_page_size = getpagesize();
+
+    /* Don't use getpagesize(), since it's not constant across Linux
+     * kernel versions on some architectures (for example PPC). FIXME:
+     * possibly the same should be done on other architectures too.
+     */
+    os_vm_page_size = BACKEND_PAGE_SIZE;
 
     /* KLUDGE: Disable memory randomization on new Linux kernels
      * by setting a personality flag and re-executing. (We need
@@ -384,11 +398,11 @@ sigsegv_handler(int signal, siginfo_t *info, void* void_context)
 #ifdef LISP_FEATURE_GENCGC
     if (!gencgc_handle_wp_violation(addr))
 #else
-    if (!interrupt_maybe_gc(signal, info, context))
+    if (!cheneygc_handle_wp_violation(context, addr))
 #endif
         if (!handle_guard_page_triggered(context, addr))
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
-            arrange_return_to_lisp_function(context, SymbolFunction(MEMORY_FAULT_ERROR));
+            lisp_memory_fault_error(context, addr);
 #else
             interrupt_handle_now(signal, info, context);
 #endif
