@@ -156,3 +156,49 @@
                      (+ x0 x2 x4 x6) (+ x1 x3 x5 x7)
                      (+ x0 x3 x4 x7) (+ x1 x2 x5 x6)
                      (+ x0 x1 x6 x7) (+ x2 x3 x4 x5)))))))
+
+
+#-x86-64
+(with-test (:name :nan-comparisons)
+  (sb-int:with-float-traps-masked (:invalid)
+    (macrolet ((test (form)
+                 (let ((nform (subst '(/ 0.0 0.0) 'nan form)))
+                   `(progn
+                      (assert (eval ',nform))
+                      (assert (eval `(let ((nan (/ 0.0 0.0)))
+                                       ,',form)))
+                      (assert (funcall
+                               (compile nil `(lambda () ,',nform))))
+                      (assert (funcall
+                               (compile nil `(lambda (nan) ,',form))
+                               (/ 0.0 0.0)))))))
+      (test (/= nan nan))
+      (test (/= nan nan nan))
+      (test (/= 1.0 nan 2.0 nan))
+      (test (/= nan 1.0 2.0 nan))
+      (test (not (= nan 1.0)))
+      (test (not (= nan nan)))
+      (test (not (= nan nan nan)))
+      (test (not (= 1.0 nan)))
+      (test (not (= nan 1.0)))
+      (test (not (= 1.0 1.0 nan)))
+      (test (not (= 1.0 nan 1.0)))
+      (test (not (= nan 1.0 1.0)))
+      (test (not (>= nan nan)))
+      (test (not (>= nan 1.0)))
+      (test (not (>= 1.0 nan)))
+      (test (not (>= 1.0 nan 0.0)))
+      (test (not (>= 1.0 0.0 nan)))
+      (test (not (>= nan 1.0 0.0)))
+      (test (not (<= nan nan)))
+      (test (not (<= nan 1.0)))
+      (test (not (<= 1.0 nan)))
+      (test (not (<= 1.0 nan 2.0)))
+      (test (not (<= 1.0 2.0 nan)))
+      (test (not (<= nan 1.0 2.0)))
+      (test (not (< nan nan)))
+      (test (not (< -1.0 nan)))
+      (test (not (< nan 1.0)))
+      (test (not (> nan nan)))
+      (test (not (> -1.0 nan)))
+      (test (not (> nan 1.0))))))
