@@ -35,7 +35,8 @@
   #+(and ppc linux) 8
   #+x86-64 16
   #+mips 8
-  #+x86 4
+  #+(and x86 (not darwin)) 4
+  #+(and x86 darwin) 16
   #-(or x86 x86-64 mips (and ppc (or darwin linux))) (error "Unknown platform"))
 
 ;;;; Build the offset-tool as regular excutable, and run it with
@@ -43,7 +44,8 @@
 ;;;; number.
 
 (run "cc"
-     #+(and linux (or x86-64 ppc)) "-fPIC"
+     #+(and (or linux freebsd) (or x86-64 ppc)) "-fPIC"
+     #+(and x86-64 darwin) "-arch" #+(and x86-64 darwin) "x86_64"
      "stack-alignment-offset.c" "-o" "stack-alignment-offset")
 
 (defparameter *good-offset*
@@ -53,7 +55,8 @@
 ;;;; Build the tool again, this time as a shared object, and load it
 
 (run "cc" "stack-alignment-offset.c"
-     #+(and linux (or x86-64 ppc)) "-fPIC"
+     #+(and (or linux freebsd) (or x86-64 ppc)) "-fPIC"
+     #+(and x86-64 darwin) "-arch" #+(and x86-64 darwin) "x86_64"
      #+darwin "-bundle" #-darwin "-shared"
      "-o" "stack-alignment-offset.so")
 
