@@ -145,6 +145,9 @@
 (defun random-type (n)
   `(integer ,(random n) ,(+ n (random n))))
 
+;;; FIXME: Since timeouts do not work on Windows this would loop
+;;; forever.
+#-win32
 (with-test (:name '(:hash-cache :interrupt))
   (let* ((type1 (random-type 500))
          (type2 (random-type 500))
